@@ -22,6 +22,8 @@
 #include <gtk/gtk.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
+#include "remminastringarray.h"
+#include "remminapublic.h"
 #include "remminafile.h"
 #include "remminafilemanager.h"
 #include "remminafileeditor.h"
@@ -59,6 +61,7 @@ struct _RemminaMainPriv
 
     gchar *selected_filename;
     gchar *selected_name;
+    RemminaStringArray *expanded_group;
 };
 
 static void
@@ -93,13 +96,54 @@ remmina_main_save_size (RemminaMain *remminamain)
     {
         remmina_pref.main_maximize = TRUE;
     }
-    remmina_pref_save ();
+}
+
+static void
+remmina_main_save_expanded_group_func (GtkTreeView *tree_view, GtkTreePath *path, RemminaMain *remminamain)
+{
+    GtkTreeIter iter;
+    gchar *group;
+
+    gtk_tree_model_get_iter (remminamain->priv->file_model_sort, &iter, path);
+    gtk_tree_model_get (remminamain->priv->file_model_sort, &iter, GROUP_COLUMN, &group, -1);
+    if (group)
+    {
+        remmina_string_array_add (remminamain->priv->expanded_group, group);
+        g_free (group);
+    }
+}
+
+static void
+remmina_main_save_expanded_group (RemminaMain *remminamain)
+{
+    if (GTK_IS_TREE_STORE (remminamain->priv->file_model))
+    {
+        if (remminamain->priv->expanded_group)
+        {
+            remmina_string_array_free (remminamain->priv->expanded_group);
+        }
+        remminamain->priv->expanded_group = remmina_string_array_new ();
+        gtk_tree_view_map_expanded_rows (GTK_TREE_VIEW (remminamain->priv->file_list),
+            (GtkTreeViewMappingFunc) remmina_main_save_expanded_group_func, remminamain);
+    }
 }
 
 static gboolean
 remmina_main_on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-    remmina_main_save_size (REMMINA_MAIN (widget));
+    RemminaMain *remminamain;
+
+    remminamain = REMMINA_MAIN (widget);
+
+    remmina_main_save_size (remminamain);
+
+    remmina_main_save_expanded_group (remminamain);
+    g_free (remmina_pref.expanded_group);
+    remmina_pref.expanded_group = remmina_string_array_to_string (remminamain->priv->expanded_group);
+    remmina_string_array_free (remminamain->priv->expanded_group);
+    remminamain->priv->expanded_group = NULL;
+
+    remmina_pref_save ();
     return FALSE;
 }
 
@@ -177,39 +221,124 @@ remmina_main_load_file_list_callback (gpointer data, gpointer user_data)
     gtk_list_store_append (store, &iter);
     gtk_list_store_set (store, &iter,
         PROTOCOL_COLUMN, remmina_file_get_icon_name (remminafile),
-        NAME_COLUMN, remminafile->name,
-        GROUP_COLUMN, remminafile->group,
-        SERVER_COLUMN, remminafile->server,
-        FILENAME_COLUMN, remminafile->filename,
+        NAME_COLUMN, remmina_file_get_string (remminafile, "name"),
+        GROUP_COLUMN, remmina_file_get_string (remminafile, "group"),
+        SERVER_COLUMN, remmina_file_get_string (remminafile, "server"),
+        FILENAME_COLUMN, remmina_file_get_filename (remminafile),
         -1);
 }
 
+static gboolean
+remmina_main_load_file_tree_traverse (GNode *node, GtkTreeStore *store, GtkTreeIter *parent)
+{
+    GtkTreeIter *iter;
+    RemminaGroupData *data;
+    GNode *child;
+
+    iter = NULL;
+    if (node->data)
+    {
+        data = (RemminaGroupData*)node->data;
+        iter = g_new0 (GtkTreeIter, 1);
+        gtk_tree_store_append (store, iter, parent);
+        gtk_tree_store_set (store, iter,
+            PROTOCOL_COLUMN, GTK_STOCK_DIRECTORY,
+            NAME_COLUMN, data->name,
+            GROUP_COLUMN, data->group,
+            FILENAME_COLUMN, NULL,
+            -1);
+    }
+    for (child = g_node_first_child (node); child; child = g_node_next_sibling (child))
+    {
+        remmina_main_load_file_tree_traverse (child, store, iter);
+    }
+    g_free (iter);
+    return FALSE;
+}
+
 static void
 remmina_main_load_file_tree_group (GtkTreeStore *store)
 {
-    GtkTreeIter iter;
-    gchar *groups, *ptr1, *ptr2;
+    GNode *root;
 
-    groups = remmina_file_manager_get_groups ();
-    if (groups == NULL || groups[0] == '\0') return;
+    root = remmina_file_manager_get_group_tree ();
+    remmina_main_load_file_tree_traverse (root, store, NULL);
+    remmina_file_manager_free_group_tree (root);
+}
 
-    ptr1 = groups;
-    while (ptr1)
+static void
+remmina_main_expand_group_traverse (RemminaMain *remminamain, GtkTreeIter *iter)
+{
+    GtkTreeModel *tree;
+    gboolean ret;
+    gchar *group, *filename;
+    GtkTreeIter child;
+    GtkTreePath *path;
+
+    tree = remminamain->priv->file_model_sort;
+    ret = TRUE;
+    while (ret)
     {
-        ptr2 = strchr (ptr1, ',');
-        if (ptr2) *ptr2++ = '\0';
+        gtk_tree_model_get (tree, iter, GROUP_COLUMN, &group, FILENAME_COLUMN, &filename, -1);
+        if (filename == NULL)
+        {
+            if (remmina_string_array_find (remminamain->priv->expanded_group, group) >= 0)
+            {
+                path = gtk_tree_model_get_path (tree, iter);
+                gtk_tree_view_expand_row (GTK_TREE_VIEW (remminamain->priv->file_list), path, FALSE);
+                gtk_tree_path_free (path);
+            }
+            if (gtk_tree_model_iter_children (tree, &child, iter))
+            {
+                remmina_main_expand_group_traverse (remminamain, &child);
+            }
+        }
+        g_free (group);
+        g_free (filename);
 
-        gtk_tree_store_append (store, &iter, NULL);
-        gtk_tree_store_set (store, &iter,
-            PROTOCOL_COLUMN, GTK_STOCK_DIRECTORY,
-            NAME_COLUMN, ptr1,
-            FILENAME_COLUMN, NULL,
-            -1);
-
-        ptr1 = ptr2;
+        ret = gtk_tree_model_iter_next (tree, iter);
     }
+}
 
-    g_free (groups);
+static void
+remmina_main_expand_group (RemminaMain *remminamain)
+{
+    GtkTreeIter iter;
+
+    if (gtk_tree_model_get_iter_first (remminamain->priv->file_model_sort, &iter))
+    {
+        remmina_main_expand_group_traverse (remminamain, &iter);
+    }
+}
+
+static gboolean
+remmina_main_load_file_tree_find (GtkTreeModel *tree, GtkTreeIter *iter, const gchar *match_group)
+{
+    gboolean ret, match;
+    gchar *group, *filename;
+    GtkTreeIter child;
+
+    match = FALSE;
+    ret = TRUE;
+    while (ret)
+    {
+        gtk_tree_model_get (tree, iter, GROUP_COLUMN, &group, FILENAME_COLUMN, &filename, -1);
+        match = (filename == NULL && g_strcmp0 (group, match_group) == 0);
+        g_free (group);
+        g_free (filename);
+        if (match) break;
+        if (gtk_tree_model_iter_children (tree, &child, iter))
+        {
+            match = remmina_main_load_file_tree_find (tree, &child, match_group);
+            if (match)
+            {
+                memcpy (iter, &child, sizeof (GtkTreeIter));
+                break;
+            }
+        }
+        ret = gtk_tree_model_iter_next (tree, iter);
+    }
+    return match;
 }
 
 static void
@@ -219,32 +348,26 @@ remmina_main_load_file_tree_callback (gpointer data, gpointer user_data)
     GtkTreeIter iter, child;
     GtkTreeStore *store;
     RemminaFile *remminafile;
-    gboolean ret, match;
-    gchar *name, *filename;
+    gboolean found;
 
     remminafile = (RemminaFile*) data;
     remminamain = (RemminaMain*) user_data;
     store = GTK_TREE_STORE (remminamain->priv->file_model);
 
-    ret = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
-    match = FALSE;
-    while (ret)
+    found = FALSE;
+    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter))
     {
-        gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, NAME_COLUMN, &name, FILENAME_COLUMN, &filename, -1);
-        match = (filename == NULL && g_strcmp0 (name, remminafile->group) == 0);
-        g_free (name);
-        g_free (filename);
-        if (match) break;
-        ret = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
+        found = remmina_main_load_file_tree_find (GTK_TREE_MODEL (store), &iter,
+            remmina_file_get_string (remminafile, "group"));
     }
 
-    gtk_tree_store_append (store, &child, (match ? &iter : NULL));
+    gtk_tree_store_append (store, &child, (found ? &iter : NULL));
     gtk_tree_store_set (store, &child,
         PROTOCOL_COLUMN, remmina_file_get_icon_name (remminafile),
-        NAME_COLUMN, remminafile->name,
-        GROUP_COLUMN, remminafile->group,
-        SERVER_COLUMN, remminafile->server,
-        FILENAME_COLUMN, remminafile->filename,
+        NAME_COLUMN, remmina_file_get_string (remminafile, "name"),
+        GROUP_COLUMN, remmina_file_get_string (remminafile, "group"),
+        SERVER_COLUMN, remmina_file_get_string (remminafile, "server"),
+        FILENAME_COLUMN, remmina_file_get_filename (remminafile),
         -1);
 }
 
@@ -280,9 +403,9 @@ remmina_main_filter_visible_func (GtkTreeModel *model, GtkTreeIter *iter, Remmin
             -1);
         if (g_strcmp0 (protocol, GTK_STOCK_DIRECTORY) != 0)
         {
-            s = g_ascii_strdown (name, -1); g_free (name); name = s;
-            s = g_ascii_strdown (group, -1); g_free (group); group = s;
-            s = g_ascii_strdown (server, -1); g_free (server); server = s;
+            s = g_ascii_strdown (name ? name : "", -1); g_free (name); name = s;
+            s = g_ascii_strdown (group ? group : "", -1); g_free (group); group = s;
+            s = g_ascii_strdown (server ? server : "", -1); g_free (server); server = s;
             result = (strstr (name, text) || strstr (server, text) || strstr (group, text));
         }
         g_free (protocol);
@@ -325,7 +448,7 @@ remmina_main_select_file (RemminaMain *remminamain, const gchar *filename)
 }
 
 static void
-remmina_main_load_files (RemminaMain *remminamain)
+remmina_main_load_files (RemminaMain *remminamain, gboolean refresh)
 {
     GtkTreeModel *filter;
     GtkTreeModel *sort;
@@ -333,6 +456,11 @@ remmina_main_load_files (RemminaMain *remminamain)
     gchar buf[200];
     guint context_id;
 
+    if (refresh)
+    {
+        remmina_main_save_expanded_group (remminamain);
+    }
+
     switch (remmina_pref.view_file_mode)
     {
 
@@ -362,11 +490,11 @@ remmina_main_load_files (RemminaMain *remminamain)
     remminamain->priv->file_model_sort = sort;
 
     gtk_tree_view_set_model (GTK_TREE_VIEW (remminamain->priv->file_list), sort);
-    gtk_tree_view_expand_all (GTK_TREE_VIEW (remminamain->priv->file_list));
     gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sort),
         remmina_pref.main_sort_column_id, remmina_pref.main_sort_order);
     g_signal_connect (G_OBJECT (sort), "sort-column-changed",
         G_CALLBACK (remmina_main_file_model_on_sort), remminamain);
+    remmina_main_expand_group (remminamain);
 
     if (remminamain->priv->selected_filename)
     {
@@ -380,28 +508,7 @@ remmina_main_load_files (RemminaMain *remminamain)
 }
 
 static void
-remmina_main_action_action_quick_connect_full (const gchar *protocol)
-{
-    GtkWidget *widget;
-
-    widget = remmina_file_editor_new_temp_full (NULL, protocol);
-    gtk_widget_show (widget);
-}
-
-static void
-remmina_main_action_action_quick_connect (GObject *object, RemminaMain *remminamain)
-{
-    remmina_main_action_action_quick_connect_full (NULL);
-}
-
-static void
-remmina_main_action_action_quick_connect_proto (GtkWidget *widget, const gchar *protocol)
-{
-    remmina_main_action_action_quick_connect_full (protocol);
-}
-
-static void
-remmina_main_action_action_connect (GtkAction *action, RemminaMain *remminamain)
+remmina_main_action_connection_connect (GtkAction *action, RemminaMain *remminamain)
 {
     remmina_connection_window_open_from_filename (remminamain->priv->selected_filename);
 }
@@ -411,12 +518,12 @@ remmina_main_file_editor_destroy (GtkWidget *widget, RemminaMain *remminamain)
 {
     if (GTK_IS_WIDGET (remminamain))
     {
-        remmina_main_load_files (remminamain);
+        remmina_main_load_files (remminamain, TRUE);
     }
 }
 
 static void
-remmina_main_action_edit_new (GtkAction *action, RemminaMain *remminamain)
+remmina_main_action_connection_new (GtkAction *action, RemminaMain *remminamain)
 {
     GtkWidget *widget;
 
@@ -426,7 +533,7 @@ remmina_main_action_edit_new (GtkAction *action, RemminaMain *remminamain)
 }
 
 static void
-remmina_main_action_edit_copy (GtkAction *action, RemminaMain *remminamain)
+remmina_main_action_connection_copy (GtkAction *action, RemminaMain *remminamain)
 {
     GtkWidget *widget;
 
@@ -441,7 +548,7 @@ remmina_main_action_edit_copy (GtkAction *action, RemminaMain *remminamain)
 }
 
 static void
-remmina_main_action_edit_edit (GtkAction *action, RemminaMain *remminamain)
+remmina_main_action_connection_edit (GtkAction *action, RemminaMain *remminamain)
 {
     GtkWidget *widget;
 
@@ -456,7 +563,7 @@ remmina_main_action_edit_edit (GtkAction *action, RemminaMain *remminamain)
 }
 
 static void
-remmina_main_action_edit_delete (GtkAction *action, RemminaMain *remminamain)
+remmina_main_action_connection_delete (GtkAction *action, RemminaMain *remminamain)
 {
     GtkWidget *dialog;
 
@@ -468,7 +575,7 @@ remmina_main_action_edit_delete (GtkAction *action, RemminaMain *remminamain)
     if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
     {
         g_unlink (remminamain->priv->selected_filename);
-        remmina_main_load_files (remminamain);
+        remmina_main_load_files (remminamain, TRUE);
     }
     gtk_widget_destroy (dialog);
     remmina_main_clear_selection_data (remminamain);
@@ -484,7 +591,7 @@ remmina_main_action_edit_preferences (GtkAction *action, RemminaMain *remminamai
 }
 
 static void
-remmina_main_action_action_quit (GtkAction *action, RemminaMain *remminamain)
+remmina_main_action_connection_close (GtkAction *action, RemminaMain *remminamain)
 {
     gtk_widget_destroy (GTK_WIDGET (remminamain));
 }
@@ -588,7 +695,7 @@ remmina_main_action_view_file_mode (GtkRadioAction *action, GtkRadioAction *curr
 {
     remmina_pref.view_file_mode = gtk_radio_action_get_current_value (action);
     remmina_pref_save ();
-    remmina_main_load_files (remminamain);
+    remmina_main_load_files (remminamain, TRUE);
 }
 
 static void
@@ -609,9 +716,9 @@ remmina_main_import_file_list (RemminaMain *remminamain, GSList *files)
         path = (gchar*) element->data;
         plugin = remmina_plugin_manager_get_import_file_handler (path);
         if (plugin && (remminafile = plugin->import_func (path)) != NULL &&
-            remminafile->name)
+            remmina_file_get_string (remminafile, "name"))
         {
-            remminafile->filename = remmina_file_generate_filename ();
+            remmina_file_generate_filename (remminafile);
             remmina_file_save_all (remminafile);
             imported = TRUE;
         }
@@ -639,7 +746,7 @@ remmina_main_import_file_list (RemminaMain *remminamain, GSList *files)
     g_string_free (err, TRUE);
     if (imported)
     {
-        remmina_main_load_files (remminamain);
+        remmina_main_load_files (remminamain, TRUE);
     }
 }
 
@@ -724,6 +831,12 @@ remmina_main_action_tools_addition (GtkAction *action, RemminaMain *remminamain)
     }
 }
 
+static void
+remmina_main_action_help_homepage (GtkAction *action, RemminaMain *remminamain)
+{
+    g_app_info_launch_default_for_uri ("http://remmina.sourceforge.net", NULL, NULL);
+}
+
 static void
 remmina_main_action_help_wiki (GtkAction *action, RemminaMain *remminamain)
 {
@@ -745,19 +858,17 @@ remmina_main_action_help_about (GtkAction *action, RemminaMain *remminamain)
 static const gchar *remmina_main_ui_xml = 
 "<ui>"
 "  <menubar name='MenuBar'>"
-"    <menu name='ActionMenu' action='Action'>"
-"      <menuitem name='ActionQuickConnectMenu' action='ActionQuickConnect'/>"
-"      <menuitem name='ActionQuickConnectProtoMenu' action='ActionQuickConnectProto'/>"
-"      <menuitem name='ActionConnectMenu' action='ActionConnect'/>"
+"    <menu name='ConnectionMenu' action='Connection'>"
+"      <menuitem name='ConnectionConnectMenu' action='ConnectionConnect'/>"
 "      <separator/>"
-"      <menuitem name='ActionQuitMenu' action='ActionQuit'/>"
+"      <menuitem name='ConnectionNewMenu' action='ConnectionNew'/>"
+"      <menuitem name='ConnectionCopyMenu' action='ConnectionCopy'/>"
+"      <menuitem name='ConnectionEditMenu' action='ConnectionEdit'/>"
+"      <menuitem name='ConnectionDeleteMenu' action='ConnectionDelete'/>"
+"      <separator/>"
+"      <menuitem name='ConnectionCloseMenu' action='ConnectionClose'/>"
 "    </menu>"
 "    <menu name='EditMenu' action='Edit'>"
-"      <menuitem name='EditNewMenu' action='EditNew'/>"
-"      <menuitem name='EditCopyMenu' action='EditCopy'/>"
-"      <menuitem name='EditEditMenu' action='EditEdit'/>"
-"      <menuitem name='EditDeleteMenu' action='EditDelete'/>"
-"      <separator/>"
 "      <menuitem name='EditPreferencesMenu' action='EditPreferences'/>"
 "    </menu>"
 "    <menu name='ViewMenu' action='View'>"
@@ -778,6 +889,7 @@ static const gchar *remmina_main_ui_xml =
 "      <menuitem name='ToolsPluginsMenu' action='ToolsPlugins'/>"
 "    </menu>"
 "    <menu name='HelpMenu' action='Help'>"
+"      <menuitem name='HelpHomepageMenu' action='HelpHomepage'/>"
 "      <menuitem name='HelpWikiMenu' action='HelpWiki'/>"
 "      <menuitem name='HelpDebugMenu' action='HelpDebug'/>"
 "      <separator/>"
@@ -785,49 +897,43 @@ static const gchar *remmina_main_ui_xml =
 "    </menu>"
 "  </menubar>"
 "  <toolbar name='ToolBar'>"
-"    <toolitem action='ActionConnect'/>"
+"    <toolitem action='ConnectionConnect'/>"
 "    <separator/>"
-"    <toolitem action='EditNew'/>"
-"    <toolitem action='EditCopy'/>"
-"    <toolitem action='EditEdit'/>"
-"    <toolitem action='EditDelete'/>"
+"    <toolitem action='ConnectionNew'/>"
+"    <toolitem action='ConnectionCopy'/>"
+"    <toolitem action='ConnectionEdit'/>"
+"    <toolitem action='ConnectionDelete'/>"
 "    <separator/>"
 "    <toolitem action='EditPreferences'/>"
 "  </toolbar>"
 "  <popup name='PopupMenu'>"
-"    <menuitem action='ActionConnect'/>"
+"    <menuitem action='ConnectionConnect'/>"
 "    <separator/>"
-"    <menuitem action='EditCopy'/>"
-"    <menuitem action='EditEdit'/>"
-"    <menuitem action='EditDelete'/>"
+"    <menuitem action='ConnectionCopy'/>"
+"    <menuitem action='ConnectionEdit'/>"
+"    <menuitem action='ConnectionDelete'/>"
 "  </popup>"
 "</ui>";
 
 static const GtkActionEntry remmina_main_ui_menu_entries[] =
 {
-    { "Action", NULL, N_("_Action") },
+    { "Connection", NULL, N_("_Connection") },
     { "Edit", NULL, N_("_Edit") },
     { "View", NULL, N_("_View") },
     { "Tools", NULL, N_("_Tools") },
     { "Help", NULL, N_("_Help") },
 
-    { "ActionQuickConnect", GTK_STOCK_JUMP_TO, N_("Quick Connect"), "<control>U",
-        N_("Open a quick connection"),
-        G_CALLBACK (remmina_main_action_action_quick_connect) },
-
-    { "ActionQuickConnectProto", NULL, N_("Quick _Connect To") },
-
-    { "EditNew", GTK_STOCK_NEW, NULL, "<control>N",
+    { "ConnectionNew", GTK_STOCK_NEW, NULL, "<control>N",
         N_("Create a new remote desktop file"),
-        G_CALLBACK (remmina_main_action_edit_new) },
+        G_CALLBACK (remmina_main_action_connection_new) },
 
     { "EditPreferences", GTK_STOCK_PREFERENCES, NULL, "<control>P",
         N_("Open the preferences dialog"),
         G_CALLBACK (remmina_main_action_edit_preferences) },
 
-    { "ActionQuit", GTK_STOCK_QUIT, NULL, "<control>Q",
-        N_("Quit Remmina"),
-        G_CALLBACK (remmina_main_action_action_quit) },
+    { "ConnectionClose", GTK_STOCK_CLOSE, NULL, "<control>X",
+        NULL,
+        G_CALLBACK (remmina_main_action_connection_close) },
 
     { "ToolsImport", NULL, N_("Import"), NULL,
         NULL,
@@ -837,6 +943,10 @@ static const GtkActionEntry remmina_main_ui_menu_entries[] =
         NULL,
         G_CALLBACK (remmina_main_action_tools_plugins) },
 
+    { "HelpHomepage", NULL, N_("Homepage"), NULL,
+        NULL,
+        G_CALLBACK (remmina_main_action_help_homepage) },
+
     { "HelpWiki", NULL, N_("Online Wiki"), NULL,
         NULL,
         G_CALLBACK (remmina_main_action_help_wiki) },
@@ -852,21 +962,21 @@ static const GtkActionEntry remmina_main_ui_menu_entries[] =
 
 static const GtkActionEntry remmina_main_ui_file_sensitive_menu_entries[] =
 {
-    { "ActionConnect", GTK_STOCK_CONNECT, NULL, "<control>O",
+    { "ConnectionConnect", GTK_STOCK_CONNECT, NULL, "<control>O",
         N_("Open the connection to the selected remote desktop file"),
-        G_CALLBACK (remmina_main_action_action_connect) },
+        G_CALLBACK (remmina_main_action_connection_connect) },
 
-    { "EditCopy", GTK_STOCK_COPY, NULL, "<control>C",
+    { "ConnectionCopy", GTK_STOCK_COPY, NULL, "<control>C",
         N_("Create a copy of the selected remote desktop file"),
-        G_CALLBACK (remmina_main_action_edit_copy) },
+        G_CALLBACK (remmina_main_action_connection_copy) },
 
-    { "EditEdit", GTK_STOCK_EDIT, NULL, "<control>E",
+    { "ConnectionEdit", GTK_STOCK_EDIT, NULL, "<control>E",
         N_("Edit the selected remote desktop file"),
-        G_CALLBACK (remmina_main_action_edit_edit) },
+        G_CALLBACK (remmina_main_action_connection_edit) },
 
-    { "EditDelete", GTK_STOCK_DELETE, NULL, "<control>D",
+    { "ConnectionDelete", GTK_STOCK_DELETE, NULL, "<control>D",
         N_("Delete the selected remote desktop file"),
-        G_CALLBACK (remmina_main_action_edit_delete) },
+        G_CALLBACK (remmina_main_action_connection_delete) },
 
     { "ToolsExport", NULL, N_("Export"), NULL,
         NULL,
@@ -912,39 +1022,17 @@ remmina_main_file_list_on_button_press (GtkWidget *widget, GdkEventButton *event
         switch (remmina_pref.default_action)
         {
         case REMMINA_ACTION_EDIT:
-            remmina_main_action_edit_edit (NULL, remminamain);
+            remmina_main_action_connection_edit (NULL, remminamain);
             break;
         case REMMINA_ACTION_CONNECT:
         default:
-            remmina_main_action_action_connect (NULL, remminamain);
+            remmina_main_action_connection_connect (NULL, remminamain);
             break;
         }
     }
     return FALSE;
 }
 
-static gboolean
-remmina_main_iterate_protocol_menu (gchar *protocol, RemminaPlugin *plugin, gpointer data)
-{
-    GtkWidget *item;
-    GtkWidget *image;
-
-    item = gtk_image_menu_item_new_with_label (plugin->description);
-    gtk_widget_show (item);
-
-    image = gtk_image_new_from_icon_name (((RemminaProtocolPlugin *)plugin)->icon_name, GTK_ICON_SIZE_MENU);
-    gtk_widget_show (image);
-    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-    gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item), TRUE);
-
-    gtk_menu_shell_append (GTK_MENU_SHELL (data), item);
-
-    g_signal_connect (G_OBJECT (item), "activate",
-        G_CALLBACK (remmina_main_action_action_quick_connect_proto), (gpointer) plugin->name);
-
-    return FALSE;
-}
-
 static void
 remmina_main_quick_search_on_icon_press (GtkEntry *entry,
     GtkEntryIconPosition icon_pos,
@@ -1031,35 +1119,52 @@ remmina_main_add_tool_plugin (gchar *name, RemminaPlugin *plugin, gpointer data)
     return FALSE;
 }
 
+static gboolean
+remmina_main_on_window_state_event (GtkWidget *widget, GdkEventWindowState *event, gpointer user_data)
+{
+    GdkScreen *screen;
+
+    screen = gdk_screen_get_default ();
+    if (remmina_pref.minimize_to_tray &&
+        (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED) != 0 &&
+        (event->new_window_state & GDK_WINDOW_STATE_ICONIFIED) != 0 &&
+        remmina_public_get_current_workspace (screen) ==
+        remmina_public_get_window_workspace (GTK_WINDOW (widget)) &&
+        gdk_screen_get_number (screen) ==
+        gdk_screen_get_number (gtk_widget_get_screen (widget))
+    )
+    {
+        gtk_widget_hide (widget);
+        return TRUE;
+    }
+    return FALSE;
+}
+
 static void
 remmina_main_init (RemminaMain *remminamain)
 {
     RemminaMainPriv *priv;
     GtkWidget *vbox;
     GtkWidget *menubar;
-    GtkWidget *menu;
-    GtkWidget *menuitem;
     GtkUIManager *uimanager;
     GtkActionGroup *action_group;
     GtkWidget *scrolledwindow;
     GtkWidget *tree;
-    GtkToolItem *toolitem;
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
     GError *error;
 
-    priv = g_new (RemminaMainPriv, 1);
+    priv = g_new0 (RemminaMainPriv, 1);
     remminamain->priv = priv;
 
-    priv->initialized = FALSE;
-    priv->selected_filename = NULL;
-    priv->selected_name = NULL;
+    remminamain->priv->expanded_group = remmina_string_array_new_from_string (remmina_pref.expanded_group);
 
     /* Create main window */
     g_signal_connect (G_OBJECT (remminamain), "delete-event", G_CALLBACK (remmina_main_on_delete_event), NULL);
     g_signal_connect (G_OBJECT (remminamain), "destroy", G_CALLBACK (remmina_main_destroy), NULL);
+    g_signal_connect (G_OBJECT (remminamain), "window-state-event", G_CALLBACK (remmina_main_on_window_state_event), NULL);
     gtk_container_set_border_width (GTK_CONTAINER (remminamain), 0);
-    gtk_window_set_title (GTK_WINDOW (remminamain), _("Remote Desktop Client"));
+    gtk_window_set_title (GTK_WINDOW (remminamain), _("Remmina Remote Desktop Client"));
     gtk_window_set_default_size (GTK_WINDOW (remminamain), remmina_pref.main_width, remmina_pref.main_height);
     gtk_window_set_position (GTK_WINDOW (remminamain), GTK_WIN_POS_CENTER);
     if (remmina_pref.main_maximize)
@@ -1117,23 +1222,6 @@ remmina_main_init (RemminaMain *remminamain)
 
     remmina_main_create_quick_search (remminamain);
 
-    /* Customized menu items and toolbar items which can't be built through standard ui manager */
-    toolitem = gtk_menu_tool_button_new_from_stock (GTK_STOCK_JUMP_TO);
-    gtk_widget_show (GTK_WIDGET (toolitem));
-    gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem), _("Quick Connect"));
-    gtk_toolbar_insert (GTK_TOOLBAR (priv->toolbar), toolitem, 0);
-    g_signal_connect (G_OBJECT (toolitem), "clicked",
-        G_CALLBACK (remmina_main_action_action_quick_connect), remminamain);
-
-    menu = gtk_menu_new ();
-    remmina_plugin_manager_for_each_plugin (REMMINA_PLUGIN_TYPE_PROTOCOL, remmina_main_iterate_protocol_menu, menu);
-    gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (toolitem), menu);
-
-    menuitem = gtk_ui_manager_get_widget (uimanager, "/MenuBar/ActionMenu/ActionQuickConnectProtoMenu");
-    menu = gtk_menu_new ();
-    remmina_plugin_manager_for_each_plugin (REMMINA_PLUGIN_TYPE_PROTOCOL, remmina_main_iterate_protocol_menu, menu);
-    gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
-
     gtk_window_add_accel_group (GTK_WINDOW (remminamain), gtk_ui_manager_get_accel_group (uimanager));
 
     gtk_action_group_set_sensitive (priv->file_sensitive_group, FALSE);
@@ -1193,7 +1281,7 @@ remmina_main_init (RemminaMain *remminamain)
     gtk_widget_show (priv->statusbar);
 
     /* Prepare the data */
-    remmina_main_load_files (remminamain);
+    remmina_main_load_files (remminamain, FALSE);
 
     /* Load the preferences */
     if (remmina_pref.hide_toolbar)
