@@ -26,10 +26,12 @@ static GPtrArray *remmina_widget_pool = NULL;
 
 static guint remmina_widget_pool_try_quit_handler = 0;
 
+static gboolean remmina_widget_pool_on_hold = FALSE;
+
 static gboolean
 remmina_widget_pool_try_quit (gpointer data)
 {
-    if (remmina_widget_pool->len == 0)
+    if (remmina_widget_pool->len == 0 && !remmina_widget_pool_on_hold)
     {
         gtk_main_quit ();
     }
@@ -116,3 +118,30 @@ remmina_widget_pool_find_by_window (GType type, GdkWindow *window)
     return NULL;
 }
 
+void
+remmina_widget_pool_hold (gboolean hold)
+{
+    remmina_widget_pool_on_hold = hold;
+    if (!hold && remmina_widget_pool_try_quit_handler == 0)
+    {
+        remmina_widget_pool_try_quit_handler = g_timeout_add (10000, remmina_widget_pool_try_quit, NULL);
+    }
+}
+
+gint
+remmina_widget_pool_foreach (RemminaWidgetPoolForEachFunc callback, gpointer data)
+{
+    GtkWidget *widget;
+    gint i;
+    gint n = 0;
+
+    if (remmina_widget_pool == NULL) return 0;
+
+    for (i = 0; i < remmina_widget_pool->len; i++)
+    {
+        widget = GTK_WIDGET (g_ptr_array_index (remmina_widget_pool, i));
+        if  (callback (widget, data)) n++;
+    }
+    return n;
+}
+
